## package versions
library("glmnet")  # glmnet_1.9-5
library("ncvreg")  # ncvreg_2.6-0
library("survival")  # survival_2.38-3
library("SIS")  # SIS_0.8-4
library("pamr")  # pamr_1.55
## Note: This standalone code is slow due to the many replications.
## It can be improved by parallelizing the code on clusters.

source("additional-functions.R")  # load the functions

## Tables 2-5 setup
## family and penalty sequence
family.seq <- c("gaussian", "binomial", "poisson", "cox")
penalty.seq <- c("SCAD", "SCAD", "lasso", "lasso")
iter.seq <- c(FALSE, rep(TRUE, 7))
varISIS.seq <- c("vanilla", "vanilla", "aggr", "cons", "vanilla", "vanilla", "cons", 
  "cons")
perm.seq <- c(rep(FALSE, 4), rep(TRUE, 4))
greedy.seq <- c(rep(FALSE, 5), TRUE, FALSE, TRUE)

tot.sim <- 100
table <- vector("list", length = 4)

## The list table contains the info of Tables 2-5.
for (case in 1:4) {
  cat("case ", case, "\n")
  family <- family.seq[case]
  penalty <- penalty.seq[case]
  results <- array(0, c(9, 5, tot.sim))
  for (randSeed in 1:tot.sim) {
    cat("randSeed ", randSeed, "\n")
    data <- gendata(case, randSeed)
    x <- data$x
    y <- data$y
    true.beta <- data$true.beta
    S <- data$S
    p <- length(true.beta)
    for (sismethodind in 1:8) {
      cat("SIS method ", sismethodind, "\n")
      prt <- proc.time()
      model <- SIS(x, y, family = family, penalty = penalty, tune = "bic", 
        iter = iter.seq[sismethodind], varISIS = varISIS.seq[sismethodind], 
        perm = perm.seq[sismethodind], greedy = greedy.seq[sismethodind], 
        seed = as.numeric(randSeed))
      running.time <- proc.time() - prt
      est.beta <- rep(0, p)
      if (case < 4) {
        est.beta[model$ix] <- model$coef.est[-1]
      } else if (case == 4) {
        est.beta[model$ix] <- model$coef.est
      }
      est.S <- model$ix
      results[sismethodind, , randSeed] <- c(summarize(est.beta, true.beta, 
        est.S, S), running.time[1])
    }
    prt <- proc.time()
    model <- tune.fit(x, y, family = family, penalty = penalty, tune = "bic")
    running.time <- proc.time() - prt
    est.beta <- rep(0, p)
    est.beta[model$ix] <- model$beta
    est.S <- model$ix
    results[9, , randSeed] <- c(summarize(est.beta, true.beta, est.S, S), running.time[1])
  }
  remed <- apply(results, c(1, 2), median)
  rersd <- apply(results, c(1, 2), IQR)/1.34
  table[[case]]$median <- remed
  table[[case]]$rsd <- rersd
}
table  # Containing all results for Tables 2-5.

## Figure 1(a): Case 1 (time vs p).
tot.sim <- 10
SIS.procedure <- vector("numeric")
full.procedure <- vector("numeric")

for (n in c(500, 1000)) {
  for (p in seq(1000, 20000, 1000)) {
    cat("n=", n, "p=", p, "\n")
    results.table <- matrix(0, nrow = tot.sim, ncol = 2)
    # Simulation of tot.sim different datasets.
    for (randSeed in 1:tot.sim) {
      
      # Generating parameter vector beta as described in Section 4.1
      # (Case 1).
      set.seed(0)
      non.zeros <- 6
      z <- abs(rnorm(non.zeros))
      signs <- rbinom(non.zeros, 1, 0.5)
      zero.entries <- which(signs == 0)
      signs[zero.entries] <- -1
      f1 <- 4 * log(n)/sqrt(n)
      f2 <- z/4
      b <- (f1 + f2) * signs
      true.beta <- rep(0, p)
      true.beta[1:non.zeros] <- b
      
      # Generating an n*p design matrix as described in Section 4.1 of the paper
      # (Case 1).
      set.seed(randSeed)
      x <- matrix(rnorm(n * p, mean = 0, sd = 1), n, p)
      y <- x[, 1:non.zeros] %*% b + rnorm(n)
      
      prt <- proc.time()
      # Model selection through the vanilla SIS (Van-SIS) approach of Figure 1(a).
      model <- SIS(x, y, family = "gaussian", penalty = "SCAD", tune = "bic", 
        iter = FALSE)
      running.time <- proc.time() - prt
      results.table[randSeed, 1] <- running.time[1]
      
      prt <- proc.time()
      # Model selection through the ncvreg approach using the full set of predictors.
      model <- tune.fit(x, y, family = "gaussian", penalty = "SCAD", tune = "bic")
      running.time <- proc.time() - prt
      results.table[randSeed, 2] <- running.time[1]
      
    }
    SIS.procedure <- c(SIS.procedure, median(results.table[, 1]))
    full.procedure <- c(full.procedure, median(results.table[, 2]))
  }
}
SIS.procedure
full.procedure

par(mgp = c(2.7, 1, 0))
plot(log10(seq(1000, 20000, 1000)), log10(SIS.procedure[1:20]), ylim = c(-1, 6), 
  xlab = expression(paste(log[10], "(p)")), ylab = expression(paste(log[10], "(T)")), 
  main = "Time vs p (Case 1)", col = "black", type = "l", lty = 1, lwd = 3, cex.lab = 1.4, 
  cex.axis = 1.4, cex.main = 1.4, cex.sub = 1.4)
axis(2, pos = 0)
lines(log10(seq(1000, 20000, 1000)), log10(full.procedure[1:20]), col = "blue", lty = 4, 
  lwd = 3)
lines(log10(seq(1000, 20000, 1000)), log10(SIS.procedure[21:40]), col = "red", lty = 2, 
  lwd = 3)
lines(log10(seq(1000, 20000, 1000)), log10(full.procedure[21:40]), col = "purple", 
  lty = 3, lwd = 3)
legend("topleft", c("n=500 Van-SIS", "n=500 SCAD/ncvreg", "n=1000 Van-SIS", "n=1000 SCAD/ncvreg"), 
  lty = c(1, 4, 2, 3), lwd = c(3, 3, 3, 3), col = c("black", "blue", "red", "purple"), 
  bty = "n", seg.len = 4, cex = 1)

# Figure 1(b): Case 2 (time vs p).
tot.sim <- 10
SIS.procedure <- vector("numeric")
full.procedure <- vector("numeric")
nlist <- c(500, 1000)
plist <- seq(1000, 20000, 1000)

for (n in nlist) {
  for (p in plist) {
    cat("n=", n, "p=", p, "\n")
    results.table <- matrix(0, nrow = tot.sim, ncol = 2)
    # Simulation of tot.sim different datasets
    for (randSeed in 1:tot.sim) {
      
      # Generating parameter vector beta as described in Section 4.1
      # (Case 2).
      set.seed(0)
      non.zeros <- 6
      z <- abs(rnorm(non.zeros))
      signs <- rbinom(non.zeros, 1, 0.5)
      zero.entries <- which(signs == 0)
      signs[zero.entries] <- -1
      f1 <- 4 * log(n)/sqrt(n)
      f2 <- z/4
      b <- (f1 + f2) * signs
      true.beta <- rep(0, p)
      true.beta[1:non.zeros] <- b
      
      # Generating an n*p design matrix as described in Section 4.1 of the paper 
      # (Case 2).
      set.seed(randSeed)
      rho <- 0.5
      corrmat <- diag(rep(1 - rho, p)) + matrix(rho, p, p)
      cholmat <- chol(corrmat)
      x <- matrix(rnorm(n * p, 0, 1), n, p)
      x <- x %*% cholmat
      feta <- x[, 1:non.zeros] %*% b
      fprob <- exp(feta)/(1 + exp(feta))
      y <- rbinom(n, 1, fprob)
      
      prt <- proc.time()
      # Model selection through the vanilla ISIS (Van-ISIS) approach of Figure 1(b).
      model <- SIS(x, y, family = "binomial", penalty = "SCAD", tune = "bic")
      running.time <- proc.time() - prt
      results.table[randSeed, 1] <- running.time[1]
      
      prt <- proc.time()
      # Model selection through the ncvreg approach using the full set of predictors.
      model <- tune.fit(x, y, family = "binomial", penalty = "SCAD", tune = "bic")
      running.time <- proc.time() - prt
      results.table[randSeed, 2] <- running.time[1]
      
    }
    SIS.procedure <- c(SIS.procedure, median(results.table[, 1]))
    full.procedure <- c(full.procedure, median(results.table[, 2]))
  }
}
SIS.procedure
full.procedure

par(mgp = c(2.7, 1, 0))
plot(log10(seq(1000, 20000, 1000)), log10(SIS.procedure[1:20]), ylim = c(-1, 6), 
  xlab = expression(paste(log[10], "(p)")), ylab = expression(paste(log[10], "(T)")), 
  main = "Time vs p (Case 2)", col = "black", type = "l", lty = 1, lwd = 3, cex.lab = 1.4, 
  cex.axis = 1.4, cex.main = 1.4, cex.sub = 1.4)
lines(log10(seq(1000, 20000, 1000)), log10(full.procedure[1:20]), col = "blue", lty = 4, 
  lwd = 3)
lines(log10(seq(1000, 20000, 1000)), log10(SIS.procedure[21:40]), col = "red", lty = 2, 
  lwd = 3)
lines(log10(seq(1000, 20000, 1000)), log10(full.procedure[21:40]), col = "purple", 
  lty = 3, lwd = 3)
legend("topleft", c("n=500 Van-ISIS", "n=500 SCAD/ncvreg", "n=1000 Van-ISIS", "n=1000 SCAD/ncvreg"), 
  lty = c(1, 4, 2, 3), lwd = c(3, 3, 3, 3), col = c("black", "blue", "red", "purple"), 
  bty = "n", seg.len = 4, cex = 1)

## Figure 1(c): Case 3 (time vs p).
tot.sim <- 10
SIS.procedure <- vector("numeric")
full.procedure <- vector("numeric")

for (n in c(500, 1000)) {
  for (p in seq(1000, 20000, 1000)) {
    cat("n=", n, "p=", p, "\n")
    results.table <- matrix(0, nrow = tot.sim, ncol = 2)
    # Simulation of tot.sim different datasets.
    for (randSeed in 1:tot.sim) {
      
      # Generating parameter vector beta as described in Section 4.1
      # (Case 3).
      b <- c(0.6, 0.6, 0.6, -0.9 * sqrt(2))
      true.beta <- rep(0, p)
      true.beta[seq_along(b)] <- b
      
      # Generating an n*p design matrix as described in Section 4.1
      # (Case 3).
      set.seed(randSeed)
      rho <- 0.5
      corrmat <- diag(rep(1 - rho, p)) + matrix(rho, p, p)
      corrmat[, 4] <- sqrt(rho)
      corrmat[4, ] <- sqrt(rho)
      corrmat[4, 4] <- 1
      cholmat <- chol(corrmat)
      x <- matrix(rnorm(n * p, 0, 1), n, p)
      x <- x %*% cholmat
      myrates <- exp(x[, 1:4] %*% b)
      y <- rpois(n, myrates)
      
      prt <- proc.time()
      # Model selection through the vanilla SIS (Van-SIS) approach of Figure 1(c).
      model <- SIS(x, y, family = "poisson", penalty = "lasso", tune = "bic", 
        iter = FALSE)
      running.time <- proc.time() - prt
      results.table[randSeed, 1] <- running.time[1]
      
      prt <- proc.time()
      # Model selection through the glmnet approach using the full set of predictors.
      model <- tune.fit(x, y, family = "poisson", penalty = "lasso", tune = "bic")
      running.time <- proc.time() - prt
      results.table[randSeed, 2] <- running.time[1]
      
    }
    SIS.procedure <- c(SIS.procedure, median(results.table[, 1]))
    full.procedure <- c(full.procedure, median(results.table[, 2]))
  }
}
SIS.procedure
full.procedure

par(mgp = c(2.7, 1, 0))
plot(log10(seq(1000, 20000, 1000)), log10(SIS.procedure[1:20]), ylim = c(-1, 6), 
  xlab = expression(paste(log[10], "(p)")), ylab = expression(paste(log[10], "(T)")), 
  main = "Time vs p (Case 3)", col = "black", type = "l", lty = 1, lwd = 3, cex.lab = 1.4, 
  cex.axis = 1.4, cex.main = 1.4, cex.sub = 1.4)
lines(log10(seq(1000, 20000, 1000)), log10(full.procedure[1:20]), col = "blue", lty = 4, 
  lwd = 3)
lines(log10(seq(1000, 20000, 1000)), log10(SIS.procedure[21:40]), col = "red", lty = 2, 
  lwd = 3)
lines(log10(seq(1000, 20000, 1000)), log10(full.procedure[21:40]), col = "purple", 
  lty = 3, lwd = 3)
legend("topleft", c("n=500 Van-SIS", "n=500 LASSO/glmnet", "n=1000 Van-SIS", "n=1000 LASSO/glmnet"), 
  lty = c(1, 4, 2, 3), lwd = c(3, 3, 3, 3), col = c("black", "blue", "red", "purple"), 
  bty = "n", seg.len = 4, cex = 1)

## Figure 1(d): Case 2 (time vs n).
tot.sim <- 10
SIS.procedure <- vector("numeric")
full.procedure <- vector("numeric")

for (p in c(10000, 20000)) {
  for (n in seq(100, 2000, 100)) {
    results.table <- matrix(0, nrow = tot.sim, ncol = 2)
    # Simulation of tot.sim different datasets.
    for (randSeed in 1:tot.sim) {
      
      # Generating parameter vector beta as described in Section 4.1
      # (Case 2).
      set.seed(0)
      non.zeros <- 6
      z <- abs(rnorm(non.zeros))
      signs <- rbinom(non.zeros, 1, 0.5)
      zero.entries <- which(signs == 0)
      signs[zero.entries] <- -1
      f1 <- 4 * log(n)/sqrt(n)
      f2 <- z/4
      b <- (f1 + f2) * signs
      true.beta <- rep(0, p)
      true.beta[1:non.zeros] <- b
      
      # Generating an n*p design matrix as described in Section 4.1
      # (Case 2).
      set.seed(randSeed)
      rho <- 0.5
      corrmat <- diag(rep(1 - rho, p)) + matrix(rho, p, p)
      cholmat <- chol(corrmat)
      x <- matrix(rnorm(n * p, 0, 1), n, p)
      x <- x %*% cholmat
      feta <- x[, 1:non.zeros] %*% b
      fprob <- exp(feta)/(1 + exp(feta))
      y <- rbinom(n, 1, fprob)
      
      prt <- proc.time()
      # Model selection through the vanilla ISIS (Van-ISIS) approach of Figure 1(d).
      model <- SIS(x, y, family = "binomial", penalty = "SCAD", tune = "bic")
      running.time <- proc.time() - prt
      results.table[randSeed, 1] <- running.time[1]
      
      prt <- proc.time()
      # Model selection through the ncvreg approach using the full set of predictors.
      model <- tune.fit(x, y, family = "binomial", penalty = "SCAD", tune = "bic")
      running.time <- proc.time() - prt
      results.table[randSeed, 2] <- running.time[1]
      
    }
    SIS.procedure <- c(SIS.procedure, median(results.table[, 1]))
    full.procedure <- c(full.procedure, median(results.table[, 2]))
  }
}
SIS.procedure
full.procedure

par(mgp = c(2.7, 1, 0))
plot(log10(seq(100, 2000, 100)), log10(SIS.procedure[1:20]), ylim = c(-1, 6), xlab = expression(paste(log[10], 
  "(n)")), ylab = expression(paste(log[10], "(T)")), main = "Time vs n (Case 2)", 
  col = "black", type = "l", lty = 1, lwd = 3, cex.lab = 1.4, cex.axis = 1.4, cex.main = 1.4, 
  cex.sub = 1.4)
lines(log10(seq(100, 2000, 100)), log10(full.procedure[1:20]), col = "blue", lty = 4, 
  lwd = 3)
lines(log10(seq(100, 2000, 100)), log10(SIS.procedure[21:40]), col = "red", lty = 2, 
  lwd = 3)
lines(log10(seq(100, 2000, 100)), log10(full.procedure[21:40]), col = "purple", lty = 3, 
  lwd = 3)
legend("topleft", c("Van-ISIS with p=10,000", "SCAD/ncvreg with p=10,000", "Van-ISIS with p=20,000", 
  "SCAD/ncvreg with p=20,000"), lty = c(1, 4, 2, 3), lwd = c(3, 3, 3, 3), col = c("black", 
  "blue", "red", "purple"), bty = "n", seg.len = 4, cex = 1)


## Tables 6 and 7

## Global Parameters

source("additional-functions.R")

penalty <- "lasso"
tune <- "cv"
nsis <- 100
q <- 0.95
st <- FALSE
tot.sim <- 100
family <- "binomial"

iter.seq <- c(FALSE, rep(TRUE, 7))
varISIS.seq <- c("vanilla", "vanilla", "aggr", "cons", "vanilla", "vanilla", "cons", 
  "cons")
perm.seq <- c(rep(FALSE, 4), rep(TRUE, 4))
greedy.seq <- c(rep(FALSE, 5), TRUE, FALSE, TRUE)
nums <- matrix(c(23, 13, 38, 30, 15, 75, 28, 95), 2, 4)
datasetnames <- c("leukemia", "prostate", "lung", "nb")
data("leukemia.train", "leukemia.test", "prostate.train", "prostate.test", "lung.train", 
  "lung.test", "nbX", "nbY", package = "SIS")
n.datasets <- length(datasetnames)

for (dataind in 1:n.datasets) {
  resultarray <- array(0, c(tot.sim, 11, 3))
  if (dataind < 4) {
    datasetname <- datasetnames[dataind]
    trainname <- paste0(datasetname, ".train")
    testname <- paste0(datasetname, ".test")
    traindata <- as.matrix(get(trainname))
    testdata <- as.matrix(get(testname))
    y1 <- traindata[, ncol(traindata)]
    x1 <- traindata[, -ncol(traindata)]
    y2 <- testdata[, ncol(testdata)]
    x2 <- testdata[, -ncol(testdata)]
    x <- rbind(x1, x2)
    y <- c(y1, y2)
  } else {
    x <- nbX
    y <- nbY
  }
  for (randSeed in 1:tot.sim) {
    set.seed(randSeed)
    results <- matrix(0, 11, 3)
    # Split datasets.
    n <- dim(x)[1]
    aux <- 1:n
    ind.train1 <- sample(aux[y == 0], nums[1, dataind], replace = FALSE)
    ind.train2 <- sample(aux[y == 1], nums[2, dataind], replace = FALSE)
    ind.train <- c(ind.train1, ind.train2)
    x.train <- scale(x[ind.train, ])
    y.train <- y[ind.train]
    ind.test1 <- setdiff(aux[y == 0], ind.train1)
    ind.test2 <- setdiff(aux[y == 1], ind.train2)
    ind.test <- c(ind.test1, ind.test2)
    x.test <- scale(x[ind.test, ])
    y.test <- y[ind.test]
    
    seed.seq <- c(0, 0, rep(randSeed, 6))
    # SIS methods
    for (sismethodind in 1:8) {
      model <- SIS(x.train, y.train, family = family, penalty = penalty, tune = tune, 
        iter = iter.seq[sismethodind], varISIS = varISIS.seq[sismethodind], 
        perm = perm.seq[sismethodind], greedy = greedy.seq[sismethodind], 
        seed = seed.seq[sismethodind], standardize = st, q = q, nsis = nsis)
      train.error <- mean((y.train != predict(model, x.train, type = "class")))
      test.error <- mean((y.test != predict(model, x.test, type = "class")))
      results[sismethodind, ] <- c(train.error, test.error, length(model$ix))
    }
    
    # 10-CV Lasso
    r <- cv.glmnet(x.train, y.train, family = "binomial")
    train.error <- mean((y.train != predict(r, x.train, type = "class")))
    test.error <- mean((y.test != predict(r, x.test, type = "class")))
    results[9, ] <- c(train.error, test.error, sum(coef(r) != 0))
    
    # NSC
    re <- nscmain(t(x.train), y.train, t(x.test), y.test)
    results[10, ] <- re
    # IR
    ir <- irmain(x.train, y.train, x.test, y.test)
    
    results[11, ] <- ir
    
    resultarray[randSeed, , ] <- results
  }
  m1 <- apply(resultarray, c(2, 3), median)
  rsd1 <- apply(resultarray, c(2, 3), IQR)/1.34
  rnames1 <- c("Van-SIS", "Van-ISIS", "Var1-ISIS", "Var2-ISIS", "Perm-ISIS", "Perm-g-ISIS", 
    "Perm-var-ISIS", "Perm-var-g-ISIS", "LASSO-CV(10)", "NSC", "IR")
  xtab(m1, rsd1, rownames = rnames1)
}
